/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Michael Terry <michael.terry@canonical.com>
 */

// LightDM currently is Qt4 compatible, and so doesn't define setRoleNames.
// To use the same method of setting role name that it does, we
// set our compatibility to Qt4 here too.
#define QT_DISABLE_DEPRECATED_BEFORE QT_VERSION_CHECK(4, 0, 0)

#include "lightdm-usersmodel.h"
#include <QtCore/QDebug>
#include <QtCore/QDir>
#include <QtCore/QString>
#include <QtGui/QIcon>

using namespace QLightDM;

class Entry
{
public:
    QString username;
    QString real_name;
    QString background;
    QString layouts;
    bool is_active;
    bool has_messages;
    QString session;
    QString infographic;
};

const Entry fake_entries[] =
{
    { "anna",         "Anna Olson",   "*", 0, false, false, 0, 0 },
    { "lois",         "Lois Mcqueen", "*", 0, false, false, 0, 0 },
    { "lola",         "Lola Chang",   "*", 0, false, false, 0, 0 },
    { "toomas",       "Toomas Vilms", "*", 0, false, false, 0, 0 },
    { "empty-name",   "",             "*", 0, false, false, 0, 0 },
    { "no-password",  "Guest",        "*", 0, false, false, 0, 0 },
    { "",             "",             0,   0, false, false, 0, 0 }
};

namespace QLightDM
{
class UsersModelPrivate
{
public:
    explicit UsersModelPrivate(UsersModel *parent);
    QList<Entry> entries;

protected:
    UsersModel * const q_ptr;
    void fillEntries();

private:
    Q_DECLARE_PUBLIC(UsersModel)
};
}

UsersModelPrivate::UsersModelPrivate(UsersModel* parent)
  : q_ptr(parent)
{
}

void UsersModelPrivate::fillEntries()
{
    QDir bgdir = QDir("/usr/share/backgrounds/");
    QStringList backgrounds = bgdir.entryList(QDir::Files | QDir::NoDotAndDotDot);
    QDir infodir = QDir("/usr/share/demo-assets/shell/infographics/");
    QStringList infographics = infodir.entryList(QDir::Files | QDir::NoDotAndDotDot);

    for (int i = 0, j = 0, k = 0; fake_entries[i].username != ""; i++) {
        Entry entry = fake_entries[i];
        if (entry.background == "*") {
            if (backgrounds.isEmpty()) {
                entry.background = ""; // to avoid warnings about a 404 on "*"
            } else {
                entry.background = bgdir.filePath(backgrounds[j++]);
                if (j >= backgrounds.length()) {
                    j = 0;
                }
            }
        }
        if (!infographics.isEmpty()) {
            entry.infographic = infodir.filePath(infographics[k++]);
            if (k >= infographics.length()) {
                k = 0;
            }
        }
        entries.append(entry);
    }
}

UsersModel::UsersModel(QObject *parent) :
    QAbstractListModel(parent),
    d_ptr(new UsersModelPrivate(this))
{
    Q_D(UsersModel);
    // Extend roleNames (we want to keep the "display" role)
    QHash<int, QByteArray> roles = roleNames();
    roles[NameRole] = "name";
    roles[RealNameRole] = "realName";
    roles[LoggedInRole] = "loggedIn";
    roles[BackgroundRole] = "background";
    roles[BackgroundPathRole] = "backgroundPath";
    roles[SessionRole] = "session";
    roles[HasMessagesRole] = "hasMessages";
    roles[ImagePathRole] = "imagePath";
    roles[InfographicRole] = "infographic";
    setRoleNames(roles);
    d->fillEntries();
}

UsersModel::~UsersModel()
{
    delete d_ptr;
}

int UsersModel::rowCount(const QModelIndex &parent) const
{
    Q_D(const UsersModel);

    if (parent.isValid()) {
        return 0;
    } else { // parent is root
        return d->entries.size();
    }
}

QVariant UsersModel::data(const QModelIndex &index, int role) const
{
    Q_D(const UsersModel);

    if (!index.isValid()) {
        return QVariant();
    }

    int row = index.row();
    switch (role) {
    case Qt::DisplayRole:
        return d->entries[row].real_name;
    case Qt::DecorationRole:
        return QIcon();
    case UsersModel::NameRole:
        return d->entries[row].username;
    case UsersModel::RealNameRole:
        return d->entries[row].real_name;
    case UsersModel::SessionRole:
        return d->entries[row].session;
    case UsersModel::LoggedInRole:
        return d->entries[row].is_active;
    case UsersModel::BackgroundRole:
        return QPixmap(d->entries[row].background);
    case UsersModel::BackgroundPathRole:
        return d->entries[row].background;
    case UsersModel::HasMessagesRole:
        return d->entries[row].has_messages;
    case UsersModel::ImagePathRole:
        return "";
    case UsersModel::InfographicRole:
        return d->entries[row].infographic;
    default:
        return QVariant();
    }
}
