/*
 * Copyright 2013 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtTest 1.0
import ".."
import "../../../Greeter"
import Ubuntu.Components 0.1
import LightDM 0.1 as LightDM
import Unity.Test 0.1 as UT

Item {
    width: units.gu(120)
    height: units.gu(80)

    Greeter {
        id: greeter
        anchors.fill: parent
    }

    SignalSpy {
        id: unlockSpy
        target: greeter
        signalName: "unlocked"
    }

    SignalSpy {
        id: selectionSpy
        target: greeter
        signalName: "selected"
    }

    UT.UnityTestCase {
        name: "Greeter"
        when: windowShown

        function select_user(name) {
            // We could be anywhere in list; find target index to know which direction
            for (var i = 0; i < greeter.model.count; i++) {
                if (greeter.model.data(i, LightDM.UserRoles.NameRole) == name) {
                    break
                }
            }
            if (i == greeter.model.count) {
                return
            }
            var pathview = findChild(greeter, "userList")
            while (pathview.currentIndex != i) {
                var next = pathview.currentIndex + 1
                if (pathview.currentIndex > i) {
                    next = pathview.currentIndex - 1
                }
                var account = findChild(greeter, "username"+next)
                mouseClick(account, 1, 1)
                tryCompare(pathview, "currentIndex", next)
            }
        }

        function test_cycle_data() {
            tryCompare(findChild(greeter, "userList"), "count", 6)
            var data = new Array()
            for (var i = 1; i < greeter.model.count; i++) {
                data[i] = {tag: greeter.model.data(i, LightDM.UserRoles.NameRole), username: "username"+i, uid: i }
            }
            // We add first name last in data to guarantee a selected signal
            data[greeter.model.count] = {tag: greeter.model.data(0, LightDM.UserRoles.NameRole), username: "username0", uid: 0 }
            return data
        }

        function test_cycle(data) {
            tryCompare(findChild(greeter, "userList"), "count", 6)
            selectionSpy.clear();
            var user = findChild(greeter, data.username)
            mouseClick(user, user.width / 2, user.height / 2)
            var pathview = findChild(greeter, "userList")
            tryCompare(pathview, "currentIndex", data.uid)
            tryCompare(greeter, "locked", data.tag !== "no-password")
            tryCompare(selectionSpy, "count", 1)
        }

        function test_unlock_password() {
            select_user("lois") // to guarantee a selected signal
            unlockSpy.clear()
            select_user("lola")
            var passwordInput = findChild(greeter, "passwordInput")
            tryCompare(passwordInput, "opacity", 1)
            mouseClick(passwordInput, 1, 1)
            compare(unlockSpy.count, 0)
            typeString("password")
            keyClick(Qt.Key_Enter)
            unlockSpy.wait()
        }

        function test_unlock_wrong_password() {
            select_user("lois") // to guarantee a selected signal
            unlockSpy.clear()
            select_user("lola")
            wait(0) // spin event loop to start any pending animations
            var passwordInput = findChild(greeter, "passwordInput")
            tryCompare(passwordInput, "opacity", 1) // wait for opacity animation to be finished
            mouseClick(passwordInput, 1, 1)
            compare(unlockSpy.count, 0)
            typeString("wr0ng p4ssw0rd")
            keyClick(Qt.Key_Enter)
            compare(unlockSpy.count, 0)
        }

        function test_unlock_no_password() {
            unlockSpy.clear()
            select_user("no-password")
            var passwordInput = findChild(greeter, "passwordInput")
            tryCompare(passwordInput, "opacity", 1)
            mouseClick(passwordInput, 1, 1)
            unlockSpy.wait()
            compare(unlockSpy.count, 1)
        }

        function test_empty_name() {
            for (var i = 0; i < greeter.model.count; i++) {
                if (greeter.model.data(i, LightDM.UserRoles.NameRole) == "empty-name") {
                    compare(greeter.model.data(i, LightDM.UserRoles.RealNameRole), greeter.model.data(i, LightDM.UserRoles.NameRole))
                    return
                }
            }
            // Never found empty-name...
            throw new Error("QtQuickTest::fail")
        }

        function test_teasingArea() {
            tryCompare(greeter, "leftTeaserPressed", false)
            mousePress(greeter, units.gu(2), greeter.height - units.gu(1))
            tryCompare(greeter, "leftTeaserPressed", true)
            mouseRelease(greeter, units.gu(2), greeter.height - units.gu(1))
            tryCompare(greeter, "leftTeaserPressed", false)

            mousePress(greeter, greeter.width - units.gu(2), greeter.height - units.gu(1))
            tryCompare(greeter, "leftTeaserPressed", false)
            mouseRelease(greeter, units.gu(2), greeter.height - units.gu(1))
            tryCompare(greeter, "leftTeaserPressed", false)
        }
    }
}
